
/* TODO
   OFFLINE Handling
------- P3 -------
   Add STATS 
   handle wssid changes in fetchPagesForDocuments. -- needed when directly using docID
   
*/
/*
Major Components
------------------
PreviewrCoreConstants -> all the constants stay here
GlobalConfig -> These are the configs passed on by the ObjectiveC layer.
DocHelper -> Core of managing the DOM, adding pages html + CSS parsing the response + making requests

readyStateCheckInterval -> This is the starting point
 
 Need to include this file before
 FetchUtils.js -> This construct the XHR calls for getting docID + fetching Pages + batch logic + error handling + XHR Cancellation
 IO.js -> This makes the XHR post and Get Calls
 
*/
var PreviewrCoreConstants = {
    BUFFER_RANGE: 5, /* number of pages to be fetched above and below the current page */
    PAGE_BATCH_SIZE: 3, /* number of pages to be fetched in a single call */
    MAX_CONCURRENT_FETCHES: 6, /* number of concurrent fetches */
    MAX_PAGE_COUNTS: 10000, /* number of concurrent fetches */
    RETRY_DELAY: 1000,
    CACHE_JS_FIRST_PAGE: 'CACHE_JS',
    CACHE_JS_OTHER_PAGES: 'CACHE_JS',
    OPTIMIZE_CACHE_LOAD: false,
    REQUEST_APP_ID: 'iosMobile',
    MAX_PAGE_RETRIES: 2,
    STATNAME: {
        PAGE_STATUS:"ymdp_js_page_status"
    },
    errorMessages: {
        INTERNAL_SERVER_ERROR: 'INTERNAL_SERVER_ERROR',
        NETWORK_FAILURE: 'NETWORK_FAILURE'
    },
};

var GlobalConfig = {
    wssid: '{{WSSID}}',
    docId: '{{DOCID}}',
    mailboxId: '{{MAILBOXID}}',
    mid: '{{MID}}',
    ymreqIdConstant: '{{YMREQIDCONSTANT}}',
    pid: '{{PID}}'
        /* mailboxId: 'VjJ-iqqFZfQBoC2f6W7H9pKnABgR8vIuWNqb3lyywoVmK3dL_VwSqHf7QVgYbQ0r4DxV-OZtZrw2rULBXQ-scIJXZQ',
        mid: '2_0_0_1_1_AI6+imIAABCNVqow9ALSqOfNXnY',
        ymreqIdConstant: 'dc77df40-ac2e-1544-028c-0c{{SEQNUM}}010a02',
        pid: '2' */
};

/* Response Parser and adding to the DOM */
var DocHelper = {

    docData: {
        pages: {},
        pageRetryCount: {}
    },
    scrollMetrics: {
        docHeight: 0,
        pagePositionMap: [0],
        currentPage: 1
    },

    /* parseResponse */
    parseResponse: function(responseObj) {
        var page, i, _this = this,
            isRetrying = false, totalPages,
            shouldSetCurrentPage = false,
            pageRetryCount = _this.docData.pageRetryCount;

        jsonData = responseObj.resultObj;

        if (!_this.docData.info) {
            if (!jsonData) {
                /* error */
                NativeNotifier.postMessage({
                    messageType: 'LOAD_STATUS',
                    success: false,
                    msg: 'Failed to Open docID ' + GlobalConfig.docId + ' error CODE ' + responseObj.errorCode + ' and no response'
                });
                return;
            } else if (jsonData.statuscode === 603 || !jsonData.success || !jsonData.info) {
                NativeNotifier.postMessage({
                    messageType: 'LOAD_STATUS',
                    success: false,
                    msg: 'Failed to Open docID ' + GlobalConfig.docId + ' error CODE ' + responseObj.errorCode + ' statusCode is ' + jsonData.statusCode
                });
                return;
            }
            NativeNotifier.postMessage({
                messageType: 'LOAD_STATUS',
                success: true,
                msg: 'Message Loaded successfully'
            });
            shouldSetCurrentPage = true;
        }

        if (jsonData.info) {
            _this.addDocInfo(jsonData.info);
            totalPages = _this.docData.info.num_pages;
            shouldSetCurrentPage && _this.setCurrentPage(_this.scrollMetrics.currentPage);
        }
        if (jsonData.pages) {
            for (i in jsonData.pages) {
                isRetrying = false;
                page = jsonData.pages[i];
                if (page.css) {
                    page.css = _this.htmlDecode(page.css);
                }
                if (page.html) {
                    page.html = _this.htmlDecode(page.html);
                } else if (!page.html && (!pageRetryCount[i] || pageRetryCount[i] < PreviewrCoreConstants.MAX_PAGE_RETRIES)) {
                    FetchUtils.fetchPreviewData({
                        reqPageNumber: parseInt(i.replace('page','')),
                        pageCount: totalPages
                    }, {
                        callback: function(respObj) {
                            _this.parseResponse(respObj);
                        }
                    });

                    pageRetryCount[i] = pageRetryCount[i] ? (pageRetryCount[i] + 1) : 1;
                    isRetrying = true;
                }
                NativeNotifier.postMessage({
                    messageType: 'STAT_JS',
                    statName: PreviewrCoreConstants.STATNAME.PAGE_STATUS,
                    msgDict: {'pageStatus': page.statuscode,
                              'pageNumber': i,
                              'isRetrying': isRetrying,
                              'docID': GlobalConfig.docId,
                              'mailboxID':GlobalConfig.mailboxId}
                });
            }
            _this.addDocPages(jsonData.pages);
        }
    },

    addDocInfoFromCache: function(infoObj) {
        var _this = this,
            docData = _this.docData;

        if (!docData.info) {
            docData.info = infoObj;
        }
    },

    addDocInfo: function(infoObj) {
        var _this = this,
            docData = _this.docData;

        if (!docData.info) {
            docData.info = infoObj;
            this.loadDocument();
            NativeNotifier.postMessage({
                messageType: PreviewrCoreConstants.CACHE_JS_FIRST_PAGE,
                msg: 'GlobalConfig.docId = "' + GlobalConfig.docId + '"; DocHelper.addDocInfoFromCache(' + JSON.stringify(infoObj) + ');'
            });
        }
        setTimeout(function() {
            _this.fetchBufferPagesIfNeeded();
        }, 1);
    },

    addDocPagesFromCache: function(pageObj, shouldRenderToDOM) {
        var i, page, cssNode, pageNode, docData = this.docData,
            cssContainer = document.getElementById('cssContainer');

        for (i in pageObj) {
            page = pageObj[i];
            if (!page.html) {
                continue;
            }
            docData.pages[i] = page;

            if (page.css) {
                cssNode = document.createElement('style');
                cssNode.innerHTML = page.css;
                cssContainer.appendChild(cssNode);
            }
            if (PreviewrCoreConstants.OPTIMIZE_CACHE_LOAD && shouldRenderToDOM) {
                this.loadExistingPageNodeForPageNumber(i);
            }
        }
    },

    addDocPages: function(pageObj) {
        var i, page, cssNode, pageNode, docData = this.docData,
            shouldCache = false, existingPage, cssContainer = document.getElementById('cssContainer'),
            isFirstPageCached = (Object.keys(docData.pages).length != 0);

        for (i in pageObj) {
            existingPage = docData.pages[i];

            if (existingPage && existingPage.html) {
                continue;
            }

            page = pageObj[i];
            docData.pages[i] = page;

            if (page.css) {
                cssNode = document.createElement('style');
                cssNode.innerHTML = page.css;
                cssContainer.appendChild(cssNode);
            }

            shouldCache = true;

            this.loadExistingPageNodeForPageNumber(i);
        }

        if (shouldCache) {
            NativeNotifier.postMessage({
                messageType: isFirstPageCached ? PreviewrCoreConstants.CACHE_JS_OTHER_PAGES : PreviewrCoreConstants.CACHE_JS_FIRST_PAGE,
                msg: 'DocHelper.addDocPagesFromCache(' + JSON.stringify(pageObj) + ',' + isFirstPageCached + ');'
            });
        }
    },

    /* Add empty pages of appropriate size to the DOM. */
    loadDocument: function() {
        var docData = this.docData,
            scrollMetrics = this.scrollMetrics,
            numOfPages = docData.info.num_pages,
            containerDiv = document.getElementById('container'),
            domWidth = containerDiv.clientWidth,
            i,
            documentDimensions = docData.info.dimension,
            exceptionPages = docData.info.exceptions,
            scale, aspectRatio,
            defaultScale = domWidth / documentDimensions.w,
            defaultAspectRatio = documentDimensions.h / documentDimensions.w;

        for (i = 1; i < numOfPages + 1; i++) {
            if (exceptionPages['p' + i]) {
                aspectRatio = (exceptionPages['p' + i].h / exceptionPages['p' + i].w);
                scale = domWidth / exceptionPages['p' + i].w;
            } else {
                aspectRatio = defaultAspectRatio;
                scale = defaultScale;
            }
            containerDiv.appendChild(this.getPageNode(i, domWidth, (domWidth * aspectRatio), scale));
        }

        scrollMetrics.docHeight = containerDiv.clientHeight;
        scrollMetrics.currentPage = 1;
        scrollMetrics.currentScrollPosition = 0;
        for (i = 2; i < numOfPages + 1; i++) {
            scrollMetrics.pagePositionMap.push(document.getElementById('lipage' + i).offsetTop / scrollMetrics.docHeight);
        }
    },

    retryFetching: function() {
        var targetNode = event.currentTarget,
            parentNode = targetNode.parentNode,
            pageNumber = parentNode.getAttribute('data-pageno'),
            pageObjKey = 'page' + pageNumber;

        delete(this.docData.pages[pageObjKey]);
        this.docData.pageRetryCount[pageObjKey] = 1;

        this.loadExistingPageNodeForPageNumber(pageObjKey);
        this.fetchBufferPagesIfNeeded();
    },

    /* creates and returns a pageDOMNode */
    getPageNode: function(pageNo, width, height, scale) {
        var markup = '<li class="pageli" id="lipage' + pageNo + '" style="height: ' + height + 'px;">' +
            '<div class="page-outer-wrapper" style="width: ' + width + 'px; height: ' + height + 'px;">' +
            '<div class="page-inner-wrapper" data-width="' + width + '" data-height="' + height + '" data-scale="' + scale + '" data-pageNo="' + pageNo + '" + id="pfpage' + pageNo + '" style="transform: scale(' + scale + ');">' + this.getPageContent(pageNo, width, height, scale) +
            '</div></div></li>';
        return this.getNodeFromMarkup(markup);
    },

    loadExistingPageNodeForPageNumber: function(pageObjKey) {
        var width, height, scale, pageNo = pageObjKey.replace('page', ''),
            pageNode = document.getElementById('pfpage' + pageNo);
        if (pageNode) {
            width = parseFloat(pageNode.getAttribute('data-width'));
            height = parseFloat(pageNode.getAttribute('data-height'));
            scale = parseFloat(pageNode.getAttribute('data-scale'));
            pageNode.innerHTML = this.getPageContent(pageNo, width, height, scale);
        }
    },

    getPageContent: function(pageNo, width, height, scale) {
        var pageObj = this.docData.pages['page' + pageNo],
            content, markup, retryCount = this.docData.pageRetryCount['page' + pageNo],
            isRetrying = retryCount && retryCount < PreviewrCoreConstants.MAX_PAGE_RETRIES,
            errorContentMarkup = '<div {{extraAttr}} style="width:' + width / scale + 'px; height:' + height / scale + 'px;"><div class="viewer-message"> {{str_message}} </div> </div>';

        if (pageObj && pageObj.html) {
            content = pageObj.html;
        } else if (pageObj && !isRetrying) {
            content = errorContentMarkup.replace('{{str_message}}', '<div class="alert"> </div><div>' + Strings.str_page_failed + '</div> <div class="retry">' + Strings.str_retry + '</div>');
            content = content.replace('{{extraAttr}}', 'onclick="DocHelper.retryFetching();"');
        } else {
            content = errorContentMarkup.replace('{{str_message}}', '<div class="bg-spinner"> </div>');
            content = content.replace('{{extraAttr}}', '');
        }

        return content;
    },

    getNodeFromMarkup: function(markupHTML) {
        var tempNode = document.createElement('div');
        tempNode.innerHTML = markupHTML;
        return tempNode.children[0];
    },

    fetchBufferPagesIfNeeded: function() {
        var _this = this,
            docData = _this.docData,
            scrollMetrics = _this.scrollMetrics,
            totalPages = docData.info.num_pages,
            currentPage = scrollMetrics.currentPage,
            startRange = currentPage - PreviewrCoreConstants.BUFFER_RANGE,
            i,
            endRange = currentPage + PreviewrCoreConstants.BUFFER_RANGE;

        if (startRange <= 0) {
            startRange = 1;
        }
        if (endRange > totalPages) {
            endRange = totalPages + 1;
        }
        
        for (i = startRange; i < endRange; i++) {
            if (!docData.pages['page' + i]) {
                FetchUtils.fetchPreviewData({
                    reqPageNumber: i,
                    pageCount: totalPages
                }, {
                    callback: function(respObj) {
                        _this.parseResponse(respObj);
                    }
                });
            }
        }
    },

    htmlDecode: function(str) {
        if (!str) {
            return;
        }

        return str.replace(/&(#?[\w\d]+);?/g, function(s, entity) {
            var ch, code;

            if (entity.charAt(0) === '#') {
                code = entity.charAt(1) === 'x' ?
                    parseInt(entity.substr(2).toLowerCase(), 16) :
                    parseInt(entity.substr(1));

                if (!(isNaN(code) || code < -32768 || code > 65535)) {
                    ch = String.fromCharCode(code);
                }
            }

            return ch || s;
        });
    },

    setCurrentPage: function(newCurrentPage) {
        var totalPages = this.docData.info.num_pages;

        if (newCurrentPage <= 0) {
            newCurrentPage = 1;
        }
        this.scrollMetrics.currentPage = newCurrentPage;
        NativeNotifier.postMessage({
            messageType: 'PAGE_INFO',
            msg: Strings.str_hover_pagenavigator + ' ' + newCurrentPage + '/' + totalPages
        });
        this.fetchBufferPagesIfNeeded();
    },

    debounce: function(fn, delay) {
        var timer,
            minDelay = 10,
            _this = this;

        return function() {
            var args = arguments;

            if (timer) {
                clearTimeout(timer);
                timer = null;
            } else {
                delay = minDelay;
            }

            timer = setTimeout(function() {
                fn.apply(_this, args);
            }, delay);
        }
    },

    initializer: function(onScrollNode) {
        var _this = this,
            requestPagesCbk = function() {
                if (!_this.docData.info) {
                    FetchUtils.fetchPreviewData({
                        reqPageNumber: 1
                    }, {
                        callback: function(respObj) {
                            _this.parseResponse(respObj);
                        }
                    });
                } else {
                    _this.loadDocument();
                    FetchUtils.fetchPreviewData({
                        reqPageNumber: 1
                    });
                    NativeNotifier.postMessage({
                        messageType: 'LOAD_STATUS',
                        success: true,
                        fromCache: true,
                        msg: 'Message Loaded successfully from cache'
                    });
                }
            };


        onScrollNode.onscroll = _this.debounce(function() {
            var scrollMetrics = _this.scrollMetrics,
                scrollPosition = (document.body.scrollTop / scrollMetrics.docHeight),
                        topThreshold, i, bottomThreshold;
                for (i = 1; i < scrollMetrics.pagePositionMap.length; i++) {
                    topThreshold = scrollMetrics.pagePositionMap[i - 1];
                    bottomThreshold = scrollMetrics.pagePositionMap[i];
                    if (scrollPosition > topThreshold && scrollPosition < bottomThreshold) {
                        _this.setCurrentPage(i);
                        return;
                    }
                }
        }, 100);


        document.body.onresize = function() {
            if (_this.resizeTimeout) {
                clearTimeout(_this.resizeTimeout);
            }
            _this.resizeTimeout = setTimeout(function() {
                var scrollMetrics = _this.scrollMetrics,
                    containerDiv = document.getElementById('container'),
                    viewportNode = document.getElementById('viewportID'),
                    scrollPosition = (document.body.scrollTop / scrollMetrics.docHeight);

                containerDiv.innerHTML = '';
                _this.loadDocument();
                document.body.scrollTop = scrollPosition * scrollMetrics.docHeight;

                viewportNode.content = 'width=device-width, initial-scale=1.0, maximum-scale=1.0';
                viewportNode.content = 'width=device-width, initial-scale=1.0';
            }, 300);
        };

        if (GlobalConfig.docId === '{{DOCID}}') {
            FetchUtils.fetchDocumentID(requestPagesCbk);
        } else {
            requestPagesCbk();
        }
    }
};
