
/* Helps in fetching pages and documentID*/
var FetchUtils = {
    retry_delay: PreviewrCoreConstants.RETRY_DELAY,

    /* fetch DocumetID push it to GlobalConfig and callback will be called */
    fetchDocumentID: function(callback) {
        var url = '/ws/v3/mailboxes/@.id==' + GlobalConfig.mailboxId +
            '/messages/@.select==q?q=id:%22' + encodeURIComponent(GlobalConfig.mid) +
            '%22&appid=' + PreviewrCoreConstants.REQUEST_APP_ID + '&wssid=' + GlobalConfig.wssid +
            '&ymreqid=' + this._getYMReqID();
        IO.get(url, null, function(resp) {
            var newWssid,
                jsonBody,
                messages, attachments, attachment, i;

            try {
                jsonBody = JSON.parse(resp.text);
            } catch (e) {
                NativeNotifier.postMessage({
                    messageType: 'LOAD_STATUS',
                    success: false,
                    msg: 'No JSON response mostly network error'
                });
            }

            if (resp.status === 400) {
                newWssid = jsonBody.error.details.wssid;
                if (newWssid) {
                    GlobalConfig.wssid = newWssid;
                    NativeNotifier.postMessage({
                        messageType: 'PROGRESS',
                        completion: 0.25,
                        msg: 'Got reissued wssid'
                    });
                    FetchUtils.fetchDocumentID(callback);
                    return;
                }
            } else if (resp.status === 200) {
                messages = jsonBody && jsonBody.result && jsonBody.result.messages;

                if (messages && messages.length === 1) {
                    attachments = messages[0].attachments;
                    for (i in attachments) {
                        attachment = attachments[i];
                        if (attachment.partId === GlobalConfig.pid) {
                            GlobalConfig.docId = attachment.documentId;
                            NativeNotifier.postMessage({
                                messageType: 'PROGRESS',
                                completion: 0.5,
                                msg: 'Got document-id'
                            });

                            callback();
                            return;
                        }
                    }
                }
            }

            NativeNotifier.postMessage({
                messageType: 'LOAD_STATUS',
                success: false,
                msg: 'No docid found for mid ' + GlobalConfig.mid + ' pid ' + GlobalConfig.pid + ' mailboxID ' + GlobalConfig.mailboxId
            });

        });
    },

    /**
     * Fetches the Preview Data based on the given Info
     * @method fetchPreviewInfo
     * @param {Object} payload eg: {'reqPageNumber':10,'pageCount':69}
     * @param {Object} context eg: {'callback': function(e)}
     */
    fetchPreviewData: function(payload, context) {
        var reqPageNumber = payload.reqPageNumber,
            fetchoptions = {},
            _this = this,
            docId = GlobalConfig.docId,
            partId = payload.partId,
            sheetInfo = payload.sheetInfo,
            fetchArr = _this._fetchArr,
            xhrObj = _this._xhrObj,
            reqId, removeReqId,
            fetchoptionsLength = PreviewrCoreConstants.PAGE_BATCH_SIZE,
            maxCuncurrentFetches = PreviewrCoreConstants.MAX_CONCURRENT_FETCHES,
            password = payload.pwd,
            hashedpassword = payload.hpwd,
            pc = payload.pageCount || PreviewrCoreConstants.MAX_PAGE_COUNTS,
            chunkStartNo,
            isDivByChunkSize;

        /* if the pc value is undefined (occurs in first request) the below checks
         need to pass and hence give high value */

        if (partId) {
            fetchoptions.zip_id = partId;
        }

        if (sheetInfo) {
            docId = docId.replace(sheetInfo.name, '');
            reqPageNumber = sheetInfo.startpage + reqPageNumber - 1;
            pc = sheetInfo.actualPageCount;
        }

        isDivByChunkSize = (reqPageNumber % fetchoptionsLength === 0);
        chunkStartNo = ((Math.floor(reqPageNumber / fetchoptionsLength) +
            (isDivByChunkSize ? -1 : 0)) * fetchoptionsLength) + 1;

        /* find the fetchoptions */
        fetchoptions.start_page = reqPageNumber;
        fetchoptions.end_page = chunkStartNo + fetchoptionsLength - 1;
        fetchoptions.end_page = (fetchoptions.end_page > pc) ? pc : fetchoptions.end_page;

        if (password) {
            fetchoptions.pwd = password;
        } else if (hashedpassword) {
            fetchoptions.hpwd = hashedpassword;
        }

        if (fetchoptions.start_page > pc) {
            return;
        }

        reqId = _this._getReqIdFromRange(chunkStartNo, fetchoptions.end_page,
            docId);

        /* check in xhrObj */
        if (xhrObj[reqId]) {
            return;
        }

        /* check the length of fetchArr if more than fetchoptionsLength
         then remove the first request and add the new one */
        if (fetchArr.length === maxCuncurrentFetches) {
            removeReqId = fetchArr[0];
            /* stop XHR for removeReqId */
            xhrObj[removeReqId].xhr.abort();
            delete xhrObj[removeReqId];
            fetchArr.splice(0, 1);
        }

        /* startFetching - after Fetch remove from FetchArr */
        this._xhrFetchPreviewData(context, fetchoptions, {
            docId: docId,
            chunkStartNo: chunkStartNo
        });
    },

    _fetchArr: [],

    _xhrObj: {},

    /**
     * Gets the RequestId given a pageRange and docId
     * @method _getReqIdFromRange
     * @param {Object} range
     * @param {String} docId
     */
    _getReqIdFromRange: function(chunkStartNo, chunkEndNo, docId) {
        return chunkStartNo + '_' + chunkEndNo + '_' + docId;
    },

    /**
     * Disables retry
     * should be used when a specific xhrReq is returning success message
     * that is to be dispatched to store
     * @param {Object} xhrObj
     */
    _terminateRetry: function(xhrObj) {
        if (xhrObj) {
            xhrObj.retrycount = 0;
            xhrObj.retry = false;
        }
    },

    /**
     * sets or increments retry counts of the xhrReq
     * and based on limit enables or disables the retry flag
     * @param {Object} xhrObj
     * @param {Integer} retryLimit
     */
    _handleRetry: function(xhrObj, retryLimit) {
        if (retryLimit > 0) {
            if (xhrObj.retrycount && xhrObj.retrycount >= retryLimit) {
                xhrObj.retry = false;

                return;
            } else {
                xhrObj.retrycount = (xhrObj.retrycount) ?
                    ((xhrObj.retrycount) + 1) : 1;
                xhrObj.retry = true;
            }
        } else {
            xhrObj.retry = false;
        }
    },

    /**
     *  400 Error Codes
     *
     *  813 - Unsupported File Type - pass to store as unsupported File Type
     *  808 - Invalid Fetch Options - pass to store as try again later
     *  806/807 - Inaccessible Download URL - pass to store as Cannot preview
     *  805 - Missing Fetch Options - (Retry once) pass to store as try again later
     *  804 - Missing Document URL - (Retry once) pass to store as Cannot preview
     *  803 - Missing Document Id - (Retry once) pass to store as Cannot preview
     *  802 - Missing App Id - (Retry once) pass to store as Cannot preview
     */

    _handle400Error: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId],
            resultObj;

        try {
            resultObj = JSON.parse(resp.text);
        } catch (e) {
            resultObj = null;
        }

        if (resultObj) {
            /* should retry once for codes 802 ,803, 804, 805
             * should not retry for codes 806, 807, 808, 813, and others
             */
            switch (resultObj.code) {
                case 802:
                case 803:
                case 804:
                case 805:
                    _this._handleRetry(xhrObj, 1);
                    break;

                default:
                    if (resultObj.error && resultObj.error.details && resultObj.error.details.wssid) {
                        GlobalConfig.wssid = resultObj.error.details.wssid;
                        _this._handleRetry(xhrObj, 1);
                    } else {
                        _this._handleRetry(xhrObj, 0);
                    }

                    break;
            }
        }

        return {
            message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
            success: false
        };
    },

    /* Unknown Error */
    _handle401Error: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId];

        this._handleRetry(xhrObj, 1);

        return {
            message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
            success: false
        };
    },

    /* Not Found */
    _handle404Error: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId];

        this._handleRetry(xhrObj, 1);

        return {
            message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
            success: false
        };
    },

    /* Internal Server Error */
    _handle500Error: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId];

        this._handleRetry(xhrObj, 2);

        return {
            message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
            success: false
        };
    },

    _handleUnexpectedErrors: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId];

        _this._terminateRetry(xhrObj);

        return {
            message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
            success: false
        };
    },

    _handleNetworkFailure: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId];

        _this._terminateRetry(xhrObj);

        return {
            message: PreviewrCoreConstants.errorMessages.NETWORK_FAILURE,
            success: false,
            start_page: (xhrObj && xhrObj.fetchoptions.start_page) || 1,
            end_page: (xhrObj && xhrObj.fetchoptions.end_page) || 1
        };
    },

    /* success from JEDI but errors from docspad side
     retry a couple of times */
    _handle200Request: function(resp, reqId) {
        var _this = this,
            xhrObj = _this._xhrObj[reqId],
            resultObj;

        try {
            resultObj = JSON.parse(resp.text);
        } catch (e) {
            resultObj = null;
        }

        if (resultObj) {
            switch (resultObj.code) {
                case 800:
                case 815:
                case 816:
                    _this._terminateRetry(xhrObj);
                    resultObj.success = true;
                    break;

                case 809:
                case 811:
                    _this._handleRetry(xhrObj, 1);
                    break;

                case 812:
                    resultObj.success = false;
                    break;

                case 814:
                    _this._handleRetry(xhrObj, 10);
                    break;

                default:
                    return {
                        message: PreviewrCoreConstants.errorMessages.INTERNAL_SERVER_ERROR,
                        success: false
                    };
            }

            return resultObj;
        }
    },

    /**
     * Parses the response and dispatches the data to the store
     * @method _dispatchToStore
     * @param {String} resp
     * @param {Object} cbkObj
     */
    _dispatchToStore: function(resp, cbkObj) {
        var _this = this,
            resultObj,
            fetchArr = _this._fetchArr,
            context = cbkObj.context,
            reqId = cbkObj.reqId,
            fetchMetaData = cbkObj.fetchMetaData,
            docId = cbkObj.docId,
            fetchoptions = cbkObj.fetchoptions,
            xhrObj = _this._xhrObj;

        switch (resp.status) {
            case 400:
                resultObj = _this._handle400Error(resp, reqId);
                break;

            case 401:
                resultObj = _this._handle401Error(resp, reqId);
                break;

            case 404:
                resultObj = _this._handle404Error(resp, reqId);
                break;

            case 500:
                resultObj = _this._handle500Error(resp, reqId);
                break;

                /* do not dipatch to store if a request is cancelled */
            case 800:
                return;

            case 200:
                resultObj = _this._handle200Request(resp, reqId);
                break;

            case 0:
                resultObj = _this._handleNetworkFailure(resp, reqId);
                break;

            default:
                resultObj = _this._handleUnexpectedErrors(resp, reqId);
                break;
        }

        resultObj = resultObj || {};

        if (xhrObj[reqId] && xhrObj[reqId].retry) {
            setTimeout(function() {
                _this._xhrFetchPreviewData(context, fetchoptions, fetchMetaData, true);
            }, _this.retry_delay);
        } else {
            delete xhrObj[reqId];

            context && context.callback && context.callback({
                resultObj: resultObj,
                docId: docId,
                errorCode: resp.status + '_' + resultObj.code
            });

            fetchArr.splice(fetchArr.indexOf(reqId), 1);
        }
    },

    /**
     * Makes an XHR fetch Request using IO promise
     * @method _xhrFetchPreviewData
     * @param {Object} context
     * @param {String} range
     * @param {String} docId
     */
    _xhrFetchPreviewData: function(context, fetchoptions, fetchMetaData, retry) {
        var _this = this,
            docId = fetchMetaData.docId,
            reqId = _this._getReqIdFromRange(fetchMetaData.chunkStartNo,
                fetchoptions.end_page, docId),
            fetchArr = _this._fetchArr,
            ymreqid = _this._getYMReqID(),
            options,
            xhrObj = _this._xhrObj,
            cbkObj = {};

        if (fetchoptions.zip_id) {
            options = {
                headers: {
                    'Docs-Document-Id': docId.replace(fetchoptions.zip_id, '')
                }
            };
        } else {
            options = {
                headers: {
                    'Docs-Document-Id': docId
                }
            };
        }

        if (!retry) {
            fetchArr.push(reqId);
            xhrObj[reqId] = {};
        }

        cbkObj.context = context;
        cbkObj.reqId = reqId;
        cbkObj.docId = docId;
        cbkObj.fetchMetaData = fetchMetaData;
        cbkObj.fetchoptions = fetchoptions;

        /* client side timeout of 15 seconds */
        options.timeout = 25000;

        xhrObj[reqId].fetchoptions = fetchoptions;
        xhrObj[reqId].xhr = IO.post('/ws/docpreview/v2/fetch?wssid=' + GlobalConfig.wssid + '&appid=' + PreviewrCoreConstants.REQUEST_APP_ID + '&ymreqid=' + ymreqid,
            fetchoptions, options,
            function(resp) {
                _this._dispatchToStore(resp, cbkObj);
            });
    },

    _getYMReqID: (function() {
        var seqNum = 0;
        return function() {
            var zeroArr = ['0', '00', '000'],
                len, numStr, seqString;
            seqNum++;
            numStr = seqNum.toString();
            len = numStr.length;
            if (len >= 4) {
                seqString = numStr.slice(len - 4, len);
            } else {
                seqString = zeroArr[3 - len] + numStr;
            }
            return GlobalConfig.ymreqIdConstant.replace('{{SEQNUM}}', seqString);
        };
    })()
};
