
/* Helps in making XHR Request*/
var IO = {
    /* post Method */
    post: function(url, data, options, callback) {
        options = options || {};

        if (typeof data === 'object') {
            data = JSON.stringify(data);
            options.headers = options.headers || {};

            if (!options.headers['Content-Type']) {
                options.headers['Content-Type'] = 'application/json';
            }
        }

        return this.issueRequest({
            method: 'POST',
            url: url,
            data: data,
            options: options
        }, callback);
    },

    /* get Method */
    get: function(url, options, callback) {
        return this.issueRequest({
            method: 'GET',
            url: url,
            options: options || {}
        }, callback);
    },

    issueRequest: function(req, callback) {
        var xhr = new XMLHttpRequest(),
            timeoutHandle;

        xhr.onreadystatechange = function() {
            var readyState = xhr.readyState;
            if (readyState === 4) {
                callback({
                    text: xhr.responseText,
                    status: xhr.status,
                    url: xhr.responseURL,
                });
            }
        };

        xhr.open(req.method, req.url, true);

        Object.keys(req.options).forEach(function(key) {
            var val = req.options[key];

            if (key === 'timeout') {
                if (typeof xhr.timeout === 'undefined') {
                    timeoutHandle = setTimeout(function() {
                        xhr.abort();
                        callback({
                            text: 'timeout',
                            status: 801
                        });
                    }, val);
                } else {
                    xhr.timeout = val;
                    xhr.ontimeout = function() {
                        callback({
                            text: 'timeout',
                            status: 801
                        });
                    };
                }
            } else if (key === 'headers') {
                Object.keys(val).forEach(function(headerName) {
                    xhr.setRequestHeader(headerName, val[headerName]);
                });
            } else {
                xhr[key] = val;
            }
        });

        try {
            if (req.method === 'POST' && !req.data) {
                req.data = '';
            }
            xhr.send(req.data);
        } catch (err) {
            callback({
                text: err.message,
                status: 500
            });
        }

        return xhr;
    }
};
