if (typeof(Stationery) == "undefined") {
    Stationery = {};
}

Stationery.constants = {
    mainComposeAreaID: (typeof(COMPOSE_CONTENT_ELEMENT_ID) !== 'undefined') ? COMPOSE_CONTENT_ELEMENT_ID : 'yahoo__compose_area',
    paperlessPostThemeProviderName: 'Paperless Post',
    motifTemplateName: 'motif_template',
    rteStationeryCoreClassName: 'rte-stationery-core',
    mobstoreBaseImageSrc: 'https://s.yimg.com/nq/tictac/stationery/themes/',
    clearStationeryClassName: 'btn-clear-stationery',
    stationeryEditableBodyID: 'middleBody',
    yahooQuotedBeginQuerySelector: '.yahoo-quoted-begin',
    outlineEnhancrQuerySelector: '.enhancr-link-previews',
    tableWrapperSelector: 'table.wrapper',
    noneStationeryThemeId: 'clearStationeryID',
    coreQuotedContentQuerySelector: 'blockquote',
    unoTemplateName: 'uno_template'
};

Stationery.getCoreBodyWithTheme = function(themeId) {
    var themeObj = JSON.parse(JSON.stringify(this.stationeryThemeConfig.stationeryThemes[themeId])),
        themeVersion = JSON.parse(JSON.stringify(this.stationeryThemeConfig.themeVersions[themeId])),
        templateName = themeObj.templateName,
        mobstoreBaseUrl = this.stationeryThemeConfig.stationeryMobThmPath,
        blockQuoteNode, yahooQuotedNode, coreContent, stationeryBodyNode, outlineEnhancrNode,
        rteCloneNode = document.createElement('div');

    rteCloneNode.innerHTML = document.getElementById(Stationery.constants.mainComposeAreaID).innerHTML;
    blockQuoteNode = rteCloneNode.querySelector(Stationery.constants.coreQuotedContentQuerySelector);
    outlineEnhancrNode = rteCloneNode.querySelector(Stationery.constants.outlineEnhancrQuerySelector);

    if (blockQuoteNode) {
        blockQuoteNode.remove();
    }

    yahooQuotedNode = rteCloneNode.querySelector(Stationery.constants.yahooQuotedBeginQuerySelector);
    if (yahooQuotedNode) {
        yahooQuotedNode.remove();
    }

    if (outlineEnhancrNode) {
        outlineEnhancrNode.remove();
    }

    stationeryBodyNode = rteCloneNode.querySelector('#' + Stationery.constants.stationeryEditableBodyID);
    if (stationeryBodyNode) {
        coreContent = stationeryBodyNode.innerHTML;
    } else {
        coreContent = rteCloneNode.innerHTML;
    }

    themeObj.yahooStationery = true;
    themeObj.pplogo = themeObj.themeProviderName === "Paperless Post";
    if (typeof(LocalizedString) === 'object') {
        themeObj.msg_footer = LocalizedString.stationery_yahoo_attribution_footer;
    } else {
        themeObj.msg_footer = "Yahoo Mail Stationery";
    }
    if (themeObj.topImg_src) {
        themeObj.topImg_src = mobstoreBaseUrl + themeVersion + '/' + themeObj.uniqueThemeName + '/' + themeObj.topImg_src;
    }
    if (themeObj.bottomImg_src) {
        themeObj.bottomImg_src = mobstoreBaseUrl + themeVersion + '/' + themeObj.uniqueThemeName + '/' + themeObj.bottomImg_src;
    }
    themeObj.messageText = coreContent;
    return Mustache.render(this.templates[templateName], themeObj);

};

Stationery.getReplyForwardBody = function() {
    var rteComposeNode = document.getElementById(Stationery.constants.mainComposeAreaID),
        coreContent = '';
        yahooQuotedNode = rteComposeNode.querySelector(Stationery.constants.yahooQuotedBeginQuerySelector),
        replyForwardQuotedNode = rteComposeNode.querySelector(Stationery.constants.coreQuotedContentQuerySelector);
        outlineEnhancrNode = rteComposeNode.querySelector(Stationery.constants.outlineEnhancrQuerySelector);

    if (yahooQuotedNode) {
        coreContent += yahooQuotedNode.outerHTML;
    }

    if (replyForwardQuotedNode) {
        coreContent += replyForwardQuotedNode.outerHTML;
    }

    if (outlineEnhancrNode) {
        coreContent += outlineEnhancrNode.outerHTML;
    }

    return coreContent;
};

Stationery.applyStationery = function(themeId) {
    // construct the new Compose Body
    var _this = this,
        replyForwardBody = _this.getReplyForwardBody(),
        coreBody = _this.getCoreBodyWithTheme(themeId),
        clearStationeryButton,
        rteStationeryNode, rteComposeNode = document.getElementById(Stationery.constants.mainComposeAreaID);

    rteComposeNode.innerHTML = coreBody + replyForwardBody;

    // Attach the handlers and contentEditableFalse
    clearStationeryButton = document.querySelector('.' + Stationery.constants.clearStationeryClassName);
    clearStationeryButton.addEventListener('click', function() {
        _this.clearStationery();
    });

    rteComposeNode.contentEditable = 'false';
    rteStationeryNode = document.querySelector('.' + Stationery.constants.rteStationeryCoreClassName);
    rteStationeryNode.addEventListener('click', function(evt) {
        if (evt.target.hasAttribute('data-action')) {
            return;
        }
        var stationeryBodyNode = document.getElementById(Stationery.constants.stationeryEditableBodyID);
        if (stationeryBodyNode) {
            stationeryBodyNode.focus();
        }
    }, false);

    //ios only
    if (window.bodyUpdated) {
        bodyUpdated();
    }

    // android only
    if (window.yMailBridge) {
        yMailBridge.setStationeryId(themeId);
    }
};

Stationery.initialize = function() {
    var rteStationeryNode, mainRTENode, stationeryBodyNode, clearStationeryButton, tableWrapperNode;
    Stationery.appendStationeryStyles();
    if (this.containsStationery()) {
        rteStationeryNode = document.querySelector('.' + Stationery.constants.rteStationeryCoreClassName);
        mainRTENode = document.getElementById(Stationery.constants.mainComposeAreaID);
        stationeryBodyNode = document.getElementById(Stationery.constants.stationeryEditableBodyID);
        rteStationeryNode.contentEditable = "false";
        mainRTENode.contentEditable = "false";
        stationeryBodyNode.contentEditable = "true";

        clearStationeryButton = rteStationeryNode.querySelector('.' + Stationery.constants.clearStationeryClassName);
        if (!clearStationeryButton) {
           clearStationeryButton = document.createElement('span');
           clearStationeryButton.className = Stationery.constants.clearStationeryClassName;
           tableWrapperNode = rteStationeryNode.querySelector(Stationery.constants.tableWrapperSelector);
           tableWrapperNode.parentNode.insertBefore(clearStationeryButton, tableWrapperNode);
        };

        clearStationeryButton.addEventListener('click', function() {
            Stationery.clearStationery();
        });

        rteStationeryNode.addEventListener('click', function() {
            stationeryBodyNode.focus();
        });
    }
};

Stationery.adjustToShowStationeryMode = function() {
    document.body.scrollTop = 0;
};

Stationery.appendStationeryStyles = function() {
    var str_clearStationery = "Clear Stationery";

    if (typeof(LocalizedString) === 'object') {
        str_clearStationery = LocalizedString.stationery_clear_stationery;
    }

    styleNode = document.createElement('style');
    styleNode.innerHTML = '.rte-stationery-core .btn-clear-stationery:after { content: "' + str_clearStationery + '"; }  blockquote .rte-stationery-core .btn-clear-stationery {display: none;} ';
    document.head.appendChild(styleNode);
};

Stationery.clearStationery = function() {
    var rteStationeryNode, mainRTENode, stationeryBodyNode, clearStationeryButton, coreHtmlContent;

    if (this.containsStationery()) {
        mainRTENode = document.getElementById(Stationery.constants.mainComposeAreaID);
        rteStationeryNode = document.querySelector('.' + Stationery.constants.rteStationeryCoreClassName);
        stationeryBodyNode = document.getElementById(Stationery.constants.stationeryEditableBodyID);

        rteStationeryNode.remove();
        coreHtmlContent = stationeryBodyNode.innerHTML;
        mainRTENode.innerHTML = coreHtmlContent + mainRTENode.innerHTML;
        mainRTENode.contentEditable = "true";

        // ios only
        if (window.bodyUpdated) {
            bodyUpdated();
            setTimeout(function() {
                window.location = "ios://clear-stationery";
            }, 0);
        }

        // android only
        if (window.yMailBridge) {
            yMailBridge.setStationeryId(NONE_STATIONERY_ID);
            syncComposeContent(mainRTENode, false);
        }
    }
};

Stationery.currentAppliedTheme = function() {
    var imageNode, srcComponents;

    if (Stationery.containsStationery()) {
        imageNode = document.querySelector("img[src^='" + Stationery.constants.mobstoreBaseImageSrc + "']");
        srcComponents = imageNode.src.split("/");
        return srcComponents[srcComponents.length - 2];
    }
    return Stationery.constants.noneStationeryThemeId;
};

Stationery.containsStationery = function() {
    var mainRTENode = document.getElementById(Stationery.constants.mainComposeAreaID),
        totalStationeryCards, stationeryCardsInBlock, quotedNode;

    totalStationeryCards = mainRTENode.getElementsByClassName(Stationery.constants.rteStationeryCoreClassName).length;

    if (totalStationeryCards == 0) {
        return false;
    }

    quotedNode = mainRTENode.querySelector(Stationery.constants.coreQuotedContentQuerySelector);
    if (quotedNode) {
        stationeryCardsInBlock = quotedNode.querySelectorAll('.' + Stationery.constants.rteStationeryCoreClassName).length;
    } else {
        stationeryCardsInBlock = 0;
    }

    return (totalStationeryCards > stationeryCardsInBlock);
};

if (typeof(document) !== 'undefined') {
    document.addEventListener( "DOMContentLoaded", function() {
        Stationery.initialize();
    }, false );
}

Stationery.pickerOrder = function (isCorp, lang) {
    var returnArr = [], themeName, themeObj,
        i, themesInOrder = Stationery.templates.supportedThemesInOrder;

    for (i in themesInOrder) {
        themeName = themesInOrder[i];
        themeObj = Stationery.stationeryThemeConfig.stationeryThemes[themeName];

        if (!themeObj || (!isCorp && themeObj.restricted) || !themeObj.supportedLang_regex.test(lang)) {
            continue;
        } else {
            returnArr.push(themeName);
        }
    }

    return returnArr;
};
